@@ -25,6 +25,29 @@
 #include "wpa_common.h"
 
 
+#define MAX_STA_COUNT 2007
+
+
+/* convert floats with one decimal place to value*10 int, i.e.,
+ * "1.5" will return 15 */
+static int hostapd_config_read_int10(const char *value)
+{
+	int i, d;
+	char *pos;
+
+	i = atoi(value);
+	pos = strchr(value, '.');
+	d = 0;
+	if (pos) {
+		pos++;
+		if (*pos >= '0' && *pos <= '9')
+			d = *pos - '0';
+	}
+
+	return i * 10 + d;
+}
+
+
 static void hostapd_config_defaults_bss(struct hostapd_bss_config *bss)
 {
 	bss->radius = (struct hostapd_radius_servers *) (bss + 1);
@@ -45,6 +68,8 @@ static void hostapd_config_defaults_bss(struct hostapd_bss_config *bss)
 	bss->wpa_pairwise = WPA_CIPHER_TKIP;
 	bss->wpa_group = WPA_CIPHER_TKIP;
 
+	bss->max_num_sta = MAX_STA_COUNT;
+
 	bss->radius_server_auth_port = 1812;
 	bss->ap_max_inactivity = AP_MAX_INACTIVITY;
 	bss->eapol_version = EAPOL_VERSION;
@@ -55,6 +80,16 @@ static struct hostapd_config * hostapd_config_defaults(void)
 {
 	struct hostapd_config *conf;
 	struct hostapd_bss_config *bss;
+	int i;
+	const int aCWmin = 15, aCWmax = 1024;
+	const struct hostapd_wme_ac_params ac_bk =
+		{ aCWmin, aCWmax, 7, 0, 0 }; /* background traffic */
+	const struct hostapd_wme_ac_params ac_be =
+		{ aCWmin, aCWmax, 3, 0, 0 }; /* best effort traffic */
+	const struct hostapd_wme_ac_params ac_vi = /* video traffic */
+		{ aCWmin >> 1, aCWmin, 2, 3000 / 32, 1 };
+	const struct hostapd_wme_ac_params ac_vo = /* voice traffic */
+		{ aCWmin >> 2, aCWmin >> 1, 2, 1500 / 32, 1 };
 
 	conf = wpa_zalloc(sizeof(*conf));
 	bss = wpa_zalloc(sizeof(*bss) + sizeof(struct hostapd_radius_servers));
@@ -80,6 +115,15 @@ static struct hostapd_config * hostapd_config_defaults(void)
 	conf->bss = bss;
 
 	conf->beacon_int = 100;
+	conf->send_probe_response = 1;
+
+	for (i = 0; i < NUM_TX_QUEUES; i++)
+		conf->tx_queue[i].aifs = -1; /* use hw default */
+
+	conf->wme_ac_params[0] = ac_be;
+	conf->wme_ac_params[1] = ac_bk;
+	conf->wme_ac_params[2] = ac_vi;
+	conf->wme_ac_params[3] = ac_vo;
 
 	return conf;
 }
@@ -609,7 +653,8 @@ static int hostapd_config_parse_key_mgmt(int line, const char *value)
 		else if (strcmp(start, "WPA-EAP") == 0)
 			val |= WPA_KEY_MGMT_IEEE8021X;
 		else {
-			printf("Line %d: invalid key_mgmt '%s'", line, start);
+			printf("Line %d: invalid key_mgmt '%s'\n",
+			       line, start);
 			free(buf);
 			return -1;
 		}
@@ -621,7 +666,7 @@ static int hostapd_config_parse_key_mgmt(int line, const char *value)
 
 	free(buf);
 	if (val == 0) {
-		printf("Line %d: no key_mgmt values configured.", line);
+		printf("Line %d: no key_mgmt values configured.\n", line);
 		return -1;
 	}
 
@@ -774,14 +819,161 @@ static int hostapd_parse_rates(int **rate_list, char *val)
 }
 
 
+static int valid_cw(int cw)
+{
+	return (cw == 1 || cw == 3 || cw == 7 || cw == 15 || cw == 31 ||
+		cw == 63 || cw == 127 || cw == 255 || cw == 511 || cw == 1023);
+}
+
+
+enum {
+	IEEE80211_TX_QUEUE_DATA0 = 0, /* used for EDCA AC_VO data */
+	IEEE80211_TX_QUEUE_DATA1 = 1, /* used for EDCA AC_VI data */
+	IEEE80211_TX_QUEUE_DATA2 = 2, /* used for EDCA AC_BE data */
+	IEEE80211_TX_QUEUE_DATA3 = 3, /* used for EDCA AC_BK data */
+	IEEE80211_TX_QUEUE_DATA4 = 4,
+	IEEE80211_TX_QUEUE_AFTER_BEACON = 6,
+	IEEE80211_TX_QUEUE_BEACON = 7
+};
+
+static int hostapd_config_tx_queue(struct hostapd_config *conf, char *name,
+				   char *val)
+{
+	int num;
+	char *pos;
+	struct hostapd_tx_queue_params *queue;
+
+	/* skip 'tx_queue_' prefix */
+	pos = name + 9;
+	if (strncmp(pos, "data", 4) == 0 &&
+	    pos[4] >= '0' && pos[4] <= '9' && pos[5] == '_') {
+		num = pos[4] - '0';
+		pos += 6;
+	} else if (strncmp(pos, "after_beacon_", 13) == 0) {
+		num = IEEE80211_TX_QUEUE_AFTER_BEACON;
+		pos += 13;
+	} else if (strncmp(pos, "beacon_", 7) == 0) {
+		num = IEEE80211_TX_QUEUE_BEACON;
+		pos += 7;
+	} else {
+		printf("Unknown tx_queue name '%s'\n", pos);
+		return -1;
+	}
+
+	queue = &conf->tx_queue[num];
+
+	if (strcmp(pos, "aifs") == 0) {
+		queue->aifs = atoi(val);
+		if (queue->aifs < 0 || queue->aifs > 255) {
+			printf("Invalid AIFS value %d\n", queue->aifs);
+			return -1;
+		}
+	} else if (strcmp(pos, "cwmin") == 0) {
+		queue->cwmin = atoi(val);
+		if (!valid_cw(queue->cwmin)) {
+			printf("Invalid cwMin value %d\n", queue->cwmin);
+			return -1;
+		}
+	} else if (strcmp(pos, "cwmax") == 0) {
+		queue->cwmax = atoi(val);
+		if (!valid_cw(queue->cwmax)) {
+			printf("Invalid cwMax value %d\n", queue->cwmax);
+			return -1;
+		}
+	} else if (strcmp(pos, "burst") == 0) {
+		queue->burst = hostapd_config_read_int10(val);
+	} else {
+		printf("Unknown tx_queue field '%s'\n", pos);
+		return -1;
+	}
+
+	queue->configured = 1;
+
+	return 0;
+}
+
+
+static int hostapd_config_wme_ac(struct hostapd_config *conf, char *name,
+				   char *val)
+{
+	int num, v;
+	char *pos;
+	struct hostapd_wme_ac_params *ac;
+
+	/* skip 'wme_ac_' prefix */
+	pos = name + 7;
+	if (strncmp(pos, "be_", 3) == 0) {
+		num = 0;
+		pos += 3;
+	} else if (strncmp(pos, "bk_", 3) == 0) {
+		num = 1;
+		pos += 3;
+	} else if (strncmp(pos, "vi_", 3) == 0) {
+		num = 2;
+		pos += 3;
+	} else if (strncmp(pos, "vo_", 3) == 0) {
+		num = 3;
+		pos += 3;
+	} else {
+		printf("Unknown wme name '%s'\n", pos);
+		return -1;
+	}
+
+	ac = &conf->wme_ac_params[num];
+
+	if (strcmp(pos, "aifs") == 0) {
+		v = atoi(val);
+		if (v < 1 || v > 255) {
+			printf("Invalid AIFS value %d\n", v);
+			return -1;
+		}
+		ac->aifs = v;
+	} else if (strcmp(pos, "cwmin") == 0) {
+		v = atoi(val);
+		if (v < 0 || v > 12) {
+			printf("Invalid cwMin value %d\n", v);
+			return -1;
+		}
+		ac->cwmin = v;
+	} else if (strcmp(pos, "cwmax") == 0) {
+		v = atoi(val);
+		if (v < 0 || v > 12) {
+			printf("Invalid cwMax value %d\n", v);
+			return -1;
+		}
+		ac->cwmax = v;
+	} else if (strcmp(pos, "txop_limit") == 0) {
+		v = atoi(val);
+		if (v < 0 || v > 0xffff) {
+			printf("Invalid txop value %d\n", v);
+			return -1;
+		}
+		ac->txopLimit = v;
+	} else if (strcmp(pos, "acm") == 0) {
+		v = atoi(val);
+		if (v < 0 || v > 1) {
+			printf("Invalid acm value %d\n", v);
+			return -1;
+		}
+		ac->admission_control_mandatory = v;
+	} else {
+		printf("Unknown wme_ac_ field '%s'\n", pos);
+		return -1;
+	}
+
+	return 0;
+}
+
+
 struct hostapd_config * hostapd_config_read(const char *fname)
 {
 	struct hostapd_config *conf;
 	struct hostapd_bss_config *bss;
 	FILE *f;
 	char buf[256], *pos;
-	int line = 0, i;
+	int line = 0;
 	int errors = 0;
+	size_t i;
 
 	f = fopen(fname, "r");
 	if (f == NULL) {
@@ -1066,6 +1258,15 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 				       line);
 				errors++;
 			}
+		} else if (strcmp(buf, "max_num_sta") == 0) {
+			bss->max_num_sta = atoi(pos);
+			if (bss->max_num_sta < 0 ||
+			    bss->max_num_sta > MAX_STA_COUNT) {
+				printf("Line %d: Invalid max_num_sta=%d; "
+				       "allowed range 0..%d\n", line,
+				       bss->max_num_sta, MAX_STA_COUNT);
+				errors++;
+			}
 		} else if (strcmp(buf, "wpa") == 0) {
 			bss->wpa = atoi(pos);
 		} else if (strcmp(buf, "wpa_group_rekey") == 0) {
@@ -1140,6 +1341,10 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 		} else if (strcmp(buf, "stakey") == 0) {
 			bss->stakey = atoi(pos);
 #endif /* CONFIG_STAKEY */
+#ifdef CONFIG_PEERKEY
+		} else if (strcmp(buf, "peerkey") == 0) {
+			bss->peerkey = atoi(pos);
+#endif /* CONFIG_PEERKEY */
 		} else if (strcmp(buf, "ctrl_interface") == 0) {
 			free(bss->ctrl_interface);
 			bss->ctrl_interface = strdup(pos);
@@ -1213,6 +1418,13 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 				errors++;
 			} else
 				conf->beacon_int = val;
+		} else if (strcmp(buf, "send_probe_response") == 0) {
+			int val = atoi(pos);
+			if (val != 0 && val != 1) {
+				printf("Line %d: invalid send_probe_response "
+				       "%d (expected 0 or 1)\n", line, val);
+			} else
+				conf->send_probe_response = val;
 		} else if (strcmp(buf, "supported_rates") == 0) {
 			if (hostapd_parse_rates(&conf->supported_rates, pos)) {
 				printf("Line %d: invalid rate list\n", line);
@@ -1244,6 +1456,30 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 			}
 		} else if (strcmp(buf, "dynamic_vlan") == 0) {
 			bss->ssid.dynamic_vlan = atoi(pos);
+		} else if (strcmp(buf, "passive_scan_interval") == 0) {
+			conf->passive_scan_interval = atoi(pos);
+		} else if (strcmp(buf, "passive_scan_listen") == 0) {
+			conf->passive_scan_listen = atoi(pos);
+		} else if (strcmp(buf, "passive_scan_mode") == 0) {
+			conf->passive_scan_mode = atoi(pos);
+		} else if (strcmp(buf, "ap_table_max_size") == 0) {
+			conf->ap_table_max_size = atoi(pos);
+		} else if (strcmp(buf, "ap_table_expiration_time") == 0) {
+			conf->ap_table_expiration_time = atoi(pos);
+		} else if (strncmp(buf, "tx_queue_", 9) == 0) {
+			if (hostapd_config_tx_queue(conf, buf, pos)) {
+				printf("Line %d: invalid TX queue item\n",
+				       line);
+				errors++;
+			}
+		} else if (strcmp(buf, "wme_enabled") == 0) {
+			bss->wme_enabled = atoi(pos);
+		} else if (strncmp(buf, "wme_ac_", 7) == 0) {
+			if (hostapd_config_wme_ac(conf, buf, pos)) {
+				printf("Line %d: invalid wme ac item\n",
+				       line);
+				errors++;
+			}
 		} else {
 			printf("Line %d: unknown configuration item '%s'\n",
 			       line, buf);
@@ -1354,7 +1590,7 @@ static void hostapd_config_free_bss(struct hostapd_bss_config *conf)
 
 void hostapd_config_free(struct hostapd_config *conf)
 {
-	int i;
+	size_t i;
 
 	if (conf == NULL)
 		return;
