@@ -87,6 +87,26 @@ struct hostapd_eap_user {
 				       * nt_password_hash() */
 };
 
+
+#define NUM_TX_QUEUES 8
+
+struct hostapd_tx_queue_params {
+	int aifs;
+	int cwmin;
+	int cwmax;
+	int burst; /* maximum burst time in 0.1 ms, i.e., 10 = 1 ms */
+	int configured;
+};
+
+struct hostapd_wme_ac_params {
+	int cwmin;
+	int cwmax;
+	int aifs;
+	int txopLimit; /* in units of 32us */
+	int admission_control_mandatory;
+};
+
+
 /**
  * struct hostapd_bss_config - Per-BSS configuration
  */
@@ -117,6 +137,8 @@ struct hostapd_bss_config {
 	       HOSTAPD_DEBUG_EXCESSIVE = 4 } debug; /* debug verbosity level */
 	char *dump_log_name; /* file name for state dump (SIGUSR1) */
 
+	int max_num_sta; /* maximum number of STAs in station table */
+
 	int ieee802_1x; /* use IEEE 802.1X */
 	int eapol_version;
 	int eap_server; /* Use internal EAP server instead of external
@@ -180,6 +202,7 @@ struct hostapd_bss_config {
 	int rsn_preauth;
 	char *rsn_preauth_interfaces;
 	int stakey;
+	int peerkey;
 
 	char *ctrl_interface; /* directory for UNIX domain sockets */
 	gid_t ctrl_interface_gid;
@@ -204,6 +227,8 @@ struct hostapd_bss_config {
 
 	int ap_max_inactivity;
 	int ignore_broadcast_ssid;
+
+	int wme_enabled;
 };
 
 
@@ -223,13 +248,41 @@ struct hostapd_config {
 	size_t num_bss;
 
 	u16 beacon_int;
+	u8 send_probe_response;
 	u8 channel;
 	hostapd_hw_mode hw_mode; /* HOSTAPD_MODE_IEEE80211A, .. */
+	enum {
+		LONG_PREAMBLE = 0,
+		SHORT_PREAMBLE = 1
+	} preamble;
+	enum {
+		CTS_PROTECTION_AUTOMATIC = 0,
+		CTS_PROTECTION_FORCE_ENABLED = 1,
+		CTS_PROTECTION_FORCE_DISABLED = 2,
+		CTS_PROTECTION_AUTOMATIC_NO_OLBC = 3,
+	} cts_protection_type;
 
 	int *supported_rates;
 	int *basic_rates;
 
 	const struct driver_ops *driver;
+
+	int passive_scan_interval; /* seconds, 0 = disabled */
+	int passive_scan_listen; /* usec */
+	int passive_scan_mode;
+	int ap_table_max_size;
+	int ap_table_expiration_time;
+
+	struct hostapd_tx_queue_params tx_queue[NUM_TX_QUEUES];
+
+	/*
+	 * WME AC parameters, in same order as 802.1D, i.e.
+	 * 0 = BE (best effort)
+	 * 1 = BK (background)
+	 * 2 = VI (video)
+	 * 3 = VO (voice)
+	 */
+	struct hostapd_wme_ac_params wme_ac_params[4];
 };
 
 
