@@ -35,6 +35,7 @@
 #include "ieee802_11.h"
 #include "sta_info.h"
 #include "hostap_common.h"
+#include "hw_features.h"
 
 
 struct hostap_driver_data {
@@ -208,7 +209,7 @@ static void handle_frame(struct hostapd_data *hapd, u8 *buf, size_t len)
 		u8 *pos = buf + len - 2;
 		extra_len = (u16) pos[1] << 8 | pos[0];
 		printf("extra data in frame (elen=%d)\n", extra_len);
-		if (extra_len + 2 > len) {
+		if ((size_t) extra_len + 2 > len) {
 			printf("  extra data overflow\n");
 			return;
 		}
@@ -227,7 +228,7 @@ static void handle_frame(struct hostapd_data *hapd, u8 *buf, size_t len)
 		HOSTAPD_DEBUG(stype == WLAN_FC_STYPE_BEACON ?
 			      HOSTAPD_DEBUG_EXCESSIVE : HOSTAPD_DEBUG_VERBOSE,
 			      "MGMT\n");
-		ieee802_11_mgmt(hapd, buf, data_len, stype);
+		ieee802_11_mgmt(hapd, buf, data_len, stype, NULL);
 		break;
 	case WLAN_FC_TYPE_CTRL:
 		HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL, "CTRL\n");
@@ -337,14 +338,13 @@ static int hostap_send_eapol(void *priv, const u8 *addr, const u8 *data,
 	int res;
 
 	len = sizeof(*hdr) + sizeof(rfc1042_header) + 2 + data_len;
-	hdr = malloc(len);
+	hdr = wpa_zalloc(len);
 	if (hdr == NULL) {
 		printf("malloc() failed for hostapd_send_data(len=%lu)\n",
 		       (unsigned long) len);
 		return -1;
 	}
 
-	memset(hdr, 0, len);
 	hdr->frame_control =
 		IEEE80211_FC(WLAN_FC_TYPE_DATA, WLAN_FC_STYPE_DATA);
 	hdr->frame_control |= host_to_le16(WLAN_FC_FROMDS);
@@ -376,8 +376,8 @@ static int hostap_send_eapol(void *priv, const u8 *addr, const u8 *data,
 }
 
 
-static int hostap_set_sta_authorized(void *priv, const u8 *addr,
-				     int authorized)
+static int hostap_sta_set_flags(void *priv, const u8 *addr,
+				int flags_or, int flags_and)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
@@ -385,18 +385,9 @@ static int hostap_set_sta_authorized(void *priv, const u8 *addr,
 	memset(&param, 0, sizeof(param));
 	param.cmd = PRISM2_HOSTAPD_SET_FLAGS_STA;
 	memcpy(param.sta_addr, addr, ETH_ALEN);
-	if (authorized) {
-		param.u.set_flags_sta.flags_or = WLAN_STA_AUTHORIZED;
-		param.u.set_flags_sta.flags_and = ~0;
-	} else {
-		param.u.set_flags_sta.flags_and = ~WLAN_STA_AUTHORIZED;
-	}
-
-	if (hostapd_ioctl(drv, &param, sizeof(param))) {
-		printf("Could not set station flags for kernel driver.\n");
-		return -1;
-	}
-	return 0;
+	param.u.set_flags_sta.flags_or = flags_or;
+	param.u.set_flags_sta.flags_and = flags_and;
+	return hostapd_ioctl(drv, &param, sizeof(param));
 }
 
 
@@ -473,10 +464,9 @@ static int hostap_set_encryption(const char *ifname, void *priv,
 	int ret = 0;
 
 	blen = sizeof(*param) + key_len;
-	buf = malloc(blen);
+	buf = wpa_zalloc(blen);
 	if (buf == NULL)
 		return -1;
-	memset(buf, 0, blen);
 
 	param = (struct prism2_hostapd_param *) buf;
 	param->cmd = PRISM2_SET_ENCRYPTION;
@@ -510,10 +500,9 @@ static int hostap_get_seqnum(const char *ifname, void *priv, const u8 *addr,
 	int ret = 0;
 
 	blen = sizeof(*param) + 32;
-	buf = malloc(blen);
+	buf = wpa_zalloc(blen);
 	if (buf == NULL)
 		return -1;
-	memset(buf, 0, blen);
 
 	param = (struct prism2_hostapd_param *) buf;
 	param->cmd = PRISM2_GET_ENCRYPTION;
@@ -667,10 +656,29 @@ static int hostap_read_sta_data(void *priv,
 
 
 static int hostap_sta_add(const char *ifname, void *priv, const u8 *addr,
-			  u16 aid, u16 capability, u8 tx_supp_rates)
+			  u16 aid, u16 capability, u8 *supp_rates,
+			  size_t supp_rates_len, int flags)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
+	int tx_supp_rates = 0;
+	size_t i;
+
+#define WLAN_RATE_1M BIT(0)
+#define WLAN_RATE_2M BIT(1)
+#define WLAN_RATE_5M5 BIT(2)
+#define WLAN_RATE_11M BIT(3)
+
+	for (i = 0; i < supp_rates_len; i++) {
+		if ((supp_rates[i] & 0x7f) == 2)
+			tx_supp_rates |= WLAN_RATE_1M;
+		if ((supp_rates[i] & 0x7f) == 4)
+			tx_supp_rates |= WLAN_RATE_2M;
+		if ((supp_rates[i] & 0x7f) == 11)
+			tx_supp_rates |= WLAN_RATE_5M5;
+		if ((supp_rates[i] & 0x7f) == 22)
+			tx_supp_rates |= WLAN_RATE_11M;
+	}
 
 	memset(&param, 0, sizeof(param));
 	param.cmd = PRISM2_HOSTAPD_ADD_STA;
@@ -687,7 +695,7 @@ static int hostap_sta_remove(void *priv, const u8 *addr)
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
 
-	hostap_set_sta_authorized(drv, addr, 0);
+	hostap_sta_set_flags(drv, addr, 0, ~WLAN_STA_AUTHORIZED);
 
 	memset(&param, 0, sizeof(param));
 	param.cmd = PRISM2_HOSTAPD_REMOVE_STA;
@@ -757,11 +765,10 @@ static int hostap_set_generic_elem(void *priv,
 	if (blen < sizeof(*param))
 		blen = sizeof(*param);
 
-	param = (struct prism2_hostapd_param *) malloc(blen);
+	param = wpa_zalloc(blen);
 	if (param == NULL)
 		return -1;
 
-	memset(param, 0, blen);
 	param->cmd = PRISM2_HOSTAPD_SET_GENERIC_ELEMENT;
 	param->u.generic_elem.len = elem_len;
 	memcpy(param->u.generic_elem.data, elem, elem_len);
@@ -863,7 +870,7 @@ static void hostapd_wireless_event_rtm_newlink(struct hostap_driver_data *drv,
 	int attrlen, nlmsg_len, rta_len;
 	struct rtattr * attr;
 
-	if (len < sizeof(*ifi))
+	if (len < (int) sizeof(*ifi))
 		return;
 
 	ifi = NLMSG_DATA(h);
@@ -911,7 +918,7 @@ static void hostapd_wireless_event_receive(int sock, void *eloop_ctx,
 	}
 
 	h = (struct nlmsghdr *) buf;
-	while (left >= sizeof(*h)) {
+	while (left >= (int) sizeof(*h)) {
 		int len, plen;
 
 		len = h->nlmsg_len;
@@ -954,10 +961,9 @@ static int hostap_get_we_version(struct hostap_driver_data *drv)
 	 * structure to grow in the future.
 	 */
 	buflen = sizeof(struct iw_range) + 500;
-	range = malloc(buflen);
+	range = wpa_zalloc(buflen);
 	if (range == NULL)
 		return -1;
-	memset(range, 0, buflen);
 
 	memset(&iwr, 0, sizeof(iwr));
 	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
@@ -1033,13 +1039,12 @@ static int hostap_init(struct hostapd_data *hapd)
 {
 	struct hostap_driver_data *drv;
 
-	drv = malloc(sizeof(struct hostap_driver_data));
+	drv = wpa_zalloc(sizeof(struct hostap_driver_data));
 	if (drv == NULL) {
 		printf("Could not allocate memory for hostapd driver data\n");
 		return -1;
 	}
 
-	memset(drv, 0, sizeof(*drv));
 	drv->ops = hostap_driver_ops;
 	drv->hapd = hapd;
 	drv->ioctl_sock = drv->sock = -1;
@@ -1126,6 +1131,56 @@ static int hostap_sta_disassoc(void *priv, const u8 *addr, int reason)
 }
 
 
+static struct hostapd_hw_modes * hostap_get_hw_feature_data(void *priv,
+							    u16 *num_modes,
+							    u16 *flags)
+{
+	struct hostapd_hw_modes *mode;
+	int i, clen, rlen;
+	const short chan2freq[14] = {
+		2412, 2417, 2422, 2427, 2432, 2437, 2442,
+		2447, 2452, 2457, 2462, 2467, 2472, 2484
+	};
+
+	mode = wpa_zalloc(sizeof(struct hostapd_hw_modes));
+	if (mode == NULL)
+		return NULL;
+
+	*num_modes = 1;
+	*flags = 0;
+
+	mode->mode = HOSTAPD_MODE_IEEE80211B;
+	mode->num_channels = 14;
+	mode->num_rates = 4;
+
+	clen = mode->num_channels * sizeof(struct hostapd_channel_data);
+	rlen = mode->num_rates * sizeof(struct hostapd_rate_data);
+
+	mode->channels = wpa_zalloc(clen);
+	mode->rates = wpa_zalloc(rlen);
+	if (mode->channels == NULL || mode->rates == NULL) {
+		hostapd_free_hw_features(mode, *num_modes);
+		return NULL;
+	}
+
+	for (i = 0; i < 14; i++) {
+		mode->channels[i].chan = i + 1;
+		mode->channels[i].freq = chan2freq[i];
+	}
+
+	mode->rates[0].rate = 10;
+	mode->rates[0].flags = HOSTAPD_RATE_CCK;
+	mode->rates[1].rate = 20;
+	mode->rates[1].flags = HOSTAPD_RATE_CCK;
+	mode->rates[2].rate = 55;
+	mode->rates[2].flags = HOSTAPD_RATE_CCK;
+	mode->rates[3].rate = 110;
+	mode->rates[3].flags = HOSTAPD_RATE_CCK;
+
+	return mode;
+}
+
+
 static const struct driver_ops hostap_driver_ops = {
 	.name = "hostap",
 	.init = hostap_init,
@@ -1140,7 +1195,7 @@ static const struct driver_ops hostap_driver_ops = {
 	.set_generic_elem = hostap_set_generic_elem,
 	.read_sta_data = hostap_read_sta_data,
 	.send_eapol = hostap_send_eapol,
-	.set_sta_authorized = hostap_set_sta_authorized,
+	.sta_set_flags = hostap_sta_set_flags,
 	.sta_deauth = hostap_sta_deauth,
 	.sta_disassoc = hostap_sta_disassoc,
 	.sta_remove = hostap_sta_remove,
@@ -1150,6 +1205,7 @@ static const struct driver_ops hostap_driver_ops = {
 	.sta_add = hostap_sta_add,
 	.get_inact_sec = hostap_get_inact_sec,
 	.sta_clear_stats = hostap_sta_clear_stats,
+	.get_hw_feature_data = hostap_get_hw_feature_data,
 };
 
 
