@@ -95,6 +95,9 @@ struct driver_ops {
 	int (*set_short_slot_time)(void *priv, int value);
 	int (*set_tx_queue_params)(void *priv, int queue, int aifs, int cw_min,
 				   int cw_max, int burst_time);
+	int (*passive_scan)(void *priv, int now, int our_mode_only,
+			    int interval, int listen, int *channel,
+			    int *last_rx);
 	struct hostapd_hw_modes * (*get_hw_feature_data)(void *priv,
 							 u16 *num_modes,
 							 u16 *flags);
@@ -481,6 +484,17 @@ hostapd_set_tx_queue_params(struct hostapd_data *hapd, int queue, int aifs,
 						 cw_min, cw_max, burst_time);
 }
 
+static inline int
+hostapd_passive_scan(struct hostapd_data *hapd, int now, int our_mode_only,
+		     int interval, int listen, int *channel,
+		     int *last_rx)
+{
+	if (hapd->driver == NULL || hapd->driver->passive_scan == NULL)
+		return -1;
+	return hapd->driver->passive_scan(hapd->driver, now, our_mode_only,
+					  interval, listen, channel, last_rx);
+}
+
 static inline struct hostapd_hw_modes *
 hostapd_get_hw_feature_data(struct hostapd_data *hapd, u16 *num_modes,
 			    u16 *flags)
