@@ -324,7 +324,7 @@ bsd_sta_set_flags(void *priv, const u8 *addr, int flags_or, int flags_and)
 	/* For now, only support setting Authorized flag */
 	if (flags_or & WLAN_STA_AUTHORIZED)
 		return bsd_set_sta_authorized(priv, addr, 1);
-	if (flags_and & WLAN_STA_AUTHORIZED)
+	if (!(flags_and & WLAN_STA_AUTHORIZED))
 		return bsd_set_sta_authorized(priv, addr, 0);
 	return 0;
 }
