@@ -72,6 +72,8 @@ struct i802_driver_data {
 	struct hostapd_data *hapd;
 
 	char iface[IFNAMSIZ + 1];
+	char mgmt_iface[IFNAMSIZ + 1];
+	int mgmt_ifindex;
 	int sock; /* raw packet socket for driver access */
 	int ioctl_sock; /* socket for ioctl() use */
 	int wext_sock; /* socket for wireless events */
@@ -96,10 +98,12 @@ static int hostapd_set_iface_flags(struct i802_driver_data *drv, int dev_up)
 		return -1;
 
 	memset(&ifr, 0, sizeof(ifr));
-	snprintf(ifr.ifr_name, IFNAMSIZ, "%sap", drv->iface);
+	snprintf(ifr.ifr_name, IFNAMSIZ, "%s", drv->iface);
 
 	if (ioctl(drv->ioctl_sock, SIOCGIFFLAGS, &ifr) != 0) {
 		perror("ioctl[SIOCGIFFLAGS]");
+		wpa_printf(MSG_DEBUG, "Could not read interface flags (%s)",
+			   drv->mgmt_iface);
 		return -1;
 	}
 
@@ -115,7 +119,7 @@ static int hostapd_set_iface_flags(struct i802_driver_data *drv, int dev_up)
 
 	if (dev_up) {
 		memset(&ifr, 0, sizeof(ifr));
-		snprintf(ifr.ifr_name, IFNAMSIZ, "%sap", drv->iface);
+		snprintf(ifr.ifr_name, IFNAMSIZ, "%s", drv->mgmt_iface);
 		ifr.ifr_mtu = HOSTAPD_MTU;
 		if (ioctl(drv->ioctl_sock, SIOCSIFMTU, &ifr) != 0) {
 			perror("ioctl[SIOCSIFMTU]");
@@ -124,24 +128,6 @@ static int hostapd_set_iface_flags(struct i802_driver_data *drv, int dev_up)
 		}
 	}
 
-        memset(&ifr, 0, sizeof(ifr));
-	snprintf(ifr.ifr_name, IFNAMSIZ, "%s.11", drv->iface);
-
-	if (ioctl(drv->ioctl_sock, SIOCGIFFLAGS, &ifr) != 0) {
-		perror("ioctl[SIOCGIFFLAGS]");
-		return -1;
-	}
-
-	if (dev_up)
-		ifr.ifr_flags |= IFF_UP;
-	else
-		ifr.ifr_flags &= ~IFF_UP;
-
-	if (ioctl(drv->ioctl_sock, SIOCSIFFLAGS, &ifr) != 0) {
-		perror("ioctl[SIOCSIFFLAGS]");
-		return -1;
-	}
-
 	return 0;
 }
 
@@ -267,7 +253,17 @@ static int i802_set_rate_sets(void *priv, int *supp_rates, int *basic_rates,
 
 	param = (struct prism2_hostapd_param *) buf;
 	param->cmd = PRISM2_HOSTAPD_SET_RATE_SETS;
-	param->u.set_rate_sets.mode = mode;
+	switch (mode) {
+	case HOSTAPD_MODE_IEEE80211A:
+		param->u.set_rate_sets.mode = MODE_IEEE80211A;
+		break;
+	case HOSTAPD_MODE_IEEE80211B:
+		param->u.set_rate_sets.mode = MODE_IEEE80211B;
+		break;
+	case HOSTAPD_MODE_IEEE80211G:
+		param->u.set_rate_sets.mode = MODE_IEEE80211G;
+		break;
+	}
 	param->u.set_rate_sets.num_supported_rates = num_supp;
 	param->u.set_rate_sets.num_basic_rates = num_basic;
 	pos = (u16 *) param->u.set_rate_sets.data;
@@ -318,7 +314,38 @@ static int hostap_ioctl_prism2param(struct i802_driver_data *drv, int param,
 	return hostap_ioctl_prism2param_iface(drv->iface, drv, param, value);
 }
 
- 
+
+static int hostap_ioctl_get_prism2param_iface(const char *iface,
+					      struct i802_driver_data *drv,
+					      int param)
+{
+	struct iwreq iwr;
+	int *i;
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, iface, IFNAMSIZ);
+	i = (int *) iwr.u.name;
+	*i = param;
+
+	if (ioctl(drv->ioctl_sock, PRISM2_IOCTL_GET_PRISM2_PARAM, &iwr) < 0) {
+		char buf[128];
+		snprintf(buf, sizeof(buf),
+			 "%s: ioctl[PRISM2_IOCTL_GET_PRISM2_PARAM]", iface);
+		perror(buf);
+		return -1;
+	}
+
+	return *i;
+}
+
+
+static int hostap_ioctl_get_prism2param(struct i802_driver_data *drv,
+					int param)
+{
+	return hostap_ioctl_get_prism2param_iface(drv->iface, drv, param);
+}
+
+
 static int i802_set_ssid(void *priv, const u8 *buf, int len)
 {
 	struct i802_driver_data *drv = priv;
@@ -355,6 +382,18 @@ static int i802_set_freq(void *priv, int mode, int freq)
 	struct i802_driver_data *drv = priv;
 	struct iwreq iwr;
 
+	switch (mode) {
+	case HOSTAPD_MODE_IEEE80211A:
+		mode = MODE_IEEE80211A;
+		break;
+	case HOSTAPD_MODE_IEEE80211B:
+		mode = MODE_IEEE80211B;
+		break;
+	case HOSTAPD_MODE_IEEE80211G:
+		mode = MODE_IEEE80211G;
+		break;
+	}
+
 	hostap_ioctl_prism2param(drv, PRISM2_PARAM_NEXT_MODE, mode);
 
 	memset(&iwr, 0, sizeof(iwr));
@@ -633,7 +672,7 @@ static int i802_sta_add(const char *ifname, void *priv, const u8 *addr,
 {
 	struct i802_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
-	int len;
+	size_t len;
 
 	memset(&param, 0, sizeof(param));
 	param.cmd = PRISM2_HOSTAPD_ADD_STA;
@@ -802,7 +841,7 @@ static int i802_set_beacon(const char *ifname, void *priv,
 	memcpy(&param->u.beacon.data[0], head, head_len);
 	memcpy(&param->u.beacon.data[0] + head_len, tail, tail_len);
 
-	if (len < sizeof(*param))
+	if (len < (int) sizeof(*param))
 		len = sizeof(*param);
 	if (hostapd_ioctl_iface(ifname, drv, param, len)) {
 		printf("Could not set beacon data to kernel driver.\n");
@@ -1170,6 +1209,7 @@ static void handle_frame(struct hostapd_iface *iface, u8 *buf, size_t len,
 	int i, broadcast_bssid = 0;
 	u8 *bssid;
 	int msg_type = ntohl(fi->msg_type);
+	struct hostapd_frame_info hfi;
 
 	/* special handling for message types without IEEE 802.11 header */
 	if (msg_type == ieee80211_msg_set_aid_for_sta) {
@@ -1272,12 +1312,19 @@ static void handle_frame(struct hostapd_iface *iface, u8 *buf, size_t len,
 			       stype == WLAN_FC_STYPE_PROBE_REQ) ?
 			      HOSTAPD_DEBUG_EXCESSIVE : HOSTAPD_DEBUG_VERBOSE,
 			      "MGMT\n");
+		memset(&hfi, 0, sizeof(hfi));
+		hfi.phytype = ntohl(fi->phytype);
+		hfi.channel = ntohl(fi->channel);
+		hfi.datarate = ntohl(fi->datarate);
+		hfi.ssi_signal = ntohl(fi->ssi_signal);
+		hfi.passive_scan = ntohl(fi->msg_type) ==
+			ieee80211_msg_passive_scan;
 		if (broadcast_bssid) {
 			for (i = 0; i < iface->num_bss; i++)
 				ieee802_11_mgmt(iface->bss[i], buf, data_len,
-						stype /*, fi */);
+						stype, &hfi);
 		} else
-			ieee802_11_mgmt(hapd, buf, data_len, stype /*, fi */);
+			ieee802_11_mgmt(hapd, buf, data_len, stype, &hfi);
 		break;
 	case WLAN_FC_TYPE_CTRL:
 		/* TODO: send deauth/disassoc if not associated STA sends
@@ -1318,7 +1365,7 @@ static void handle_read(int sock, void *eloop_ctx, void *sock_ctx)
 		printf("\n");
         }
 
-	if (len < sizeof(struct ieee80211_frame_info)) {
+	if (len < (int) sizeof(struct ieee80211_frame_info)) {
 		printf("handle_read: too short (%d)\n", len);
 		return;
 	}
@@ -1344,10 +1391,9 @@ static int i802_init_sockets(struct i802_driver_data *drv)
 	struct hostapd_iface *iface = hapd->iface;
 	struct ifreq ifr;
 	struct sockaddr_ll addr;
-	u8 *ifname;
+	struct iwreq iwr;
 
 	drv->sock = drv->ioctl_sock = -1;
-	ifname = drv->iface;
 
 	drv->ioctl_sock = socket(PF_INET, SOCK_DGRAM, 0);
 	if (drv->ioctl_sock < 0) {
@@ -1355,12 +1401,34 @@ static int i802_init_sockets(struct i802_driver_data *drv)
 		return -1;
 	}
 
-        memset(&ifr, 0, sizeof(ifr));
-        snprintf(ifr.ifr_name, sizeof(ifr.ifr_name), "%sap", ifname);
-        if (ioctl(drv->ioctl_sock, SIOCGIFINDEX, &ifr) != 0) {
-		perror("ioctl(SIOCGIFINDEX)");
+	/* Enable management interface */
+	if (hostap_ioctl_prism2param(drv, PRISM2_PARAM_MGMT_IF, 1) < 0) {
+		printf("Failed to enable management interface.\n");
 		return -1;
-        }
+	}
+	drv->mgmt_ifindex =
+		hostap_ioctl_get_prism2param(drv, PRISM2_PARAM_MGMT_IF);
+	if (drv->mgmt_ifindex < 0) {
+		printf("Failed to get ifindex for the management "
+		       "interface.\n");
+		return -1;
+	}
+
+	memset(&ifr, 0, sizeof(ifr));
+	ifr.ifr_ifindex = drv->mgmt_ifindex;
+	if (ioctl(drv->ioctl_sock, SIOCGIFNAME, &ifr) != 0) {
+		perror("ioctl(SIOCGIFNAME)");
+		return -1;
+	}
+	snprintf(drv->mgmt_iface, sizeof(drv->mgmt_iface), "%s", ifr.ifr_name);
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, hapd->conf->iface, IFNAMSIZ);
+	iwr.u.mode = IW_MODE_MASTER;
+
+	if (ioctl(drv->ioctl_sock, SIOCSIWMODE, &iwr) < 0) {
+		perror("ioctl[SIOCSIWMODE]");
+	}
 
 	if (hostapd_set_iface_flags(drv, 1))
 		return -1;
@@ -1389,7 +1457,7 @@ static int i802_init_sockets(struct i802_driver_data *drv)
 	}
 
         memset(&ifr, 0, sizeof(ifr));
-        snprintf(ifr.ifr_name, sizeof(ifr.ifr_name), "%s", ifname);
+        snprintf(ifr.ifr_name, sizeof(ifr.ifr_name), "%s", drv->iface);
         if (ioctl(drv->sock, SIOCGIFHWADDR, &ifr) != 0) {
 		perror("ioctl(SIOCGIFHWADDR)");
 		return -1;
@@ -1519,9 +1587,9 @@ static void hostapd_wireless_event_rtm_newlink(struct i802_driver_data *drv,
 {
 	struct ifinfomsg *ifi;
 	int attrlen, nlmsg_len, rta_len;
-	struct rtattr * attr;
+	struct rtattr *attr;
 
-	if (len < sizeof(*ifi))
+	if (len < (int) sizeof(*ifi))
 		return;
 
 	ifi = NLMSG_DATA(h);
@@ -1569,7 +1637,7 @@ static void hostapd_wireless_event_receive(int sock, void *eloop_ctx,
 	}
 
 	h = (struct nlmsghdr *) buf;
-	while (left >= sizeof(*h)) {
+	while (left >= (int) sizeof(*h)) {
 		int len, plen;
 
 		len = h->nlmsg_len;
@@ -1740,7 +1808,7 @@ static int i802_init(struct hostapd_data *hapd)
 	hapd->driver = &drv->ops;
 
 	/* Enable the radio by default. */
-	(void) hostap_ioctl_prism2param(drv, PRISM2_PARAM_ADM_STATUS, 1);
+	(void) hostap_ioctl_prism2param(drv, PRISM2_PARAM_RADIO_ENABLED, 1);
 
 	return 0;
 
@@ -1755,7 +1823,10 @@ static void i802_deinit(void *priv)
 	struct i802_driver_data *drv = priv;
 
 	/* Disable the radio. */
-	(void) hostap_ioctl_prism2param(drv, PRISM2_PARAM_ADM_STATUS, 0);
+	(void) hostap_ioctl_prism2param(drv, PRISM2_PARAM_RADIO_ENABLED, 0);
+
+	/* Disable management interface */
+	(void) hostap_ioctl_prism2param(drv, PRISM2_PARAM_MGMT_IF, 0);
 
 	drv->hapd->driver = NULL;
 
