@@ -160,7 +160,8 @@ set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 #endif /* MADWIFI_NG */
 		int idx = op - first;
 		if (first <= op && op <= last &&
-		    idx < sizeof(opnames) / sizeof(opnames[0]) && opnames[idx])
+		    idx < (int) (sizeof(opnames) / sizeof(opnames[0])) &&
+		    opnames[idx])
 			perror(opnames[idx]);
 		else
 			perror("ioctl[unknown???]");
@@ -405,7 +406,7 @@ madwifi_sta_set_flags(void *priv, const u8 *addr, int flags_or, int flags_and)
 	/* For now, only support setting Authorized flag */
 	if (flags_or & WLAN_STA_AUTHORIZED)
 		return madwifi_set_sta_authorized(priv, addr, 1);
-	if (flags_and & WLAN_STA_AUTHORIZED)
+	if (!(flags_and & WLAN_STA_AUTHORIZED))
 		return madwifi_set_sta_authorized(priv, addr, 0);
 	return 0;
 }
@@ -916,7 +917,7 @@ madwifi_wireless_event_rtm_newlink(struct madwifi_driver_data *drv,
 	int attrlen, nlmsg_len, rta_len;
 	struct rtattr * attr;
 
-	if (len < sizeof(*ifi))
+	if (len < (int) sizeof(*ifi))
 		return;
 
 	ifi = NLMSG_DATA(h);
@@ -964,7 +965,7 @@ madwifi_wireless_event_receive(int sock, void *eloop_ctx, void *sock_ctx)
 	}
 
 	h = (struct nlmsghdr *) buf;
-	while (left >= sizeof(*h)) {
+	while (left >= (int) sizeof(*h)) {
 		int len, plen;
 
 		len = h->nlmsg_len;
