@@ -238,7 +238,7 @@ static int prism54_set_encryption(const char *ifname, void *priv,
 	} else {
 		printf("bad auth type: %s\n", alg);
 	}
-	buf = &keys->key[0];
+	buf = (u8 *) &keys->key[0];
 	keys->length = key_len;
 	keys->keyid = idx;
 	keys->options = htons(DOT11_STAKEY_OPTION_DEFAULTKEY);
@@ -292,7 +292,7 @@ static int prism54_get_seqnum(const char *ifname, void *priv, const u8 *addr,
 	if (prism54_waitpim(priv, DOT11_OID_STASC, hdr, blen, 10) <= 0) {
 		ret = -1;
 	} else {
-		if (hdr->op == htonl(PIMOP_RESPONSE)) {
+		if (hdr->op == (int) htonl(PIMOP_RESPONSE)) {
 			memcpy(seq + 2, &stasc->sc_high, ETH_ALEN);
 			memset(seq, 0, 2);
 		} else {
@@ -415,7 +415,7 @@ static int prism54_flush(void *priv)
 	hdr->oid = htonl(DOT11_OID_CLIENTS);
 	ret = send(drv->pim_sock, hdr, sizeof(*hdr) + sizeof(long), 0);
 	ret = prism54_waitpim(priv, DOT11_OID_CLIENTS, hdr, blen, 10);
-	if ((ret < 0) || (hdr->op != htonl(PIMOP_RESPONSE)) ||
+	if ((ret < 0) || (hdr->op != (int) htonl(PIMOP_RESPONSE)) ||
 	    (le_to_host32(*nsta) > 2007)) {
 		free(hdr);
 		return 0;
@@ -521,7 +521,7 @@ static int prism54_get_inact_sec(void *priv, const u8 *addr)
 		free(hdr);
 		return -1;
 	}
-	if (hdr->op != htonl(PIMOP_RESPONSE)) {
+	if (hdr->op != (int) htonl(PIMOP_RESPONSE)) {
 		printf("get_inact_sec: bad resp\n");
 		free(hdr);
 		return -1;
@@ -572,7 +572,7 @@ static void prism54_handle_probe(struct prism54_driver_data *drv,
 	struct sta_info *sta;
 	hdr = (pimdev_hdr *)buf;
 	mlme = (struct obj_mlmeex *) &hdr[1];
-	sta = ap_get_sta(drv->hapd, &mlme->address[0]);
+	sta = ap_get_sta(drv->hapd, (u8 *) &mlme->address[0]);
 	if (sta != NULL) {
 		if (sta->flags & (WLAN_STA_AUTH | WLAN_STA_ASSOC))
 			return;
@@ -598,7 +598,7 @@ static void prism54_handle_deauth(struct prism54_driver_data *drv,
 	struct sta_info *sta;
 	hdr = (pimdev_hdr *) buf;
 	mlme = (struct obj_mlme *) &hdr[1];
-	sta = ap_get_sta(drv->hapd, &mlme->address[0]);
+	sta = ap_get_sta(drv->hapd, (u8 *) &mlme->address[0]);
 	memcpy(&mlme->address[0], mac_id_get(drv, mlme->id), ETH_ALEN);
 	if (sta == NULL) {
 		return;
@@ -620,7 +620,7 @@ static void prism54_handle_disassoc(struct prism54_driver_data *drv,
 	hdr = (pimdev_hdr *) buf;
 	mlme = (struct obj_mlme *) &hdr[1];
 	memcpy(&mlme->address[0], mac_id_get(drv, mlme->id), ETH_ALEN);
-	sta = ap_get_sta(drv->hapd, &mlme->address[0]);
+	sta = ap_get_sta(drv->hapd, (u8 *) &mlme->address[0]);
 	if (sta == NULL) {
 		return;
 	}
@@ -650,7 +650,7 @@ static void prism54_handle_auth(struct prism54_driver_data *drv,
 	}
 
 	if (mlme->state == htons(DOT11_STATE_AUTHING)) {
-		sta = ap_sta_add(drv->hapd, &mlme->address[0]);
+		sta = ap_sta_add(drv->hapd, (u8 *) &mlme->address[0]);
 		if (drv->hapd->tkip_countermeasures) {
 			resp = WLAN_REASON_MICHAEL_MIC_FAILURE;
 			goto fail;
@@ -709,19 +709,19 @@ static void prism54_handle_assoc(struct prism54_driver_data *drv,
 		default:
 			break;
 	}
-	if ((mlme->state == htonl(DOT11_STATE_ASSOCING)) ||
-	    (mlme->state == htonl(DOT11_STATE_REASSOCING))) {
+	if ((mlme->state == (int) htonl(DOT11_STATE_ASSOCING)) ||
+	    (mlme->state == (int) htonl(DOT11_STATE_REASSOCING))) {
 		if (len < sizeof(pimdev_hdr) + sizeof(struct obj_mlme)) {
 			printf("bad assoc packet\n");
 			return;
 		}
 		memcpy(&mlme->address[0], mac_id_get(drv, mlme->id), ETH_ALEN);
-		sta = ap_get_sta(drv->hapd, &mlme->address[0]);
+		sta = ap_get_sta(drv->hapd, (u8 *) &mlme->address[0]);
 		if (sta == NULL) {
 			printf("cannot get sta\n");
 			return;
 		}
-		cb = &mlme->data[0];
+		cb = (u8 *) &mlme->data[0];
 		if (hdr->oid == htonl(DOT11_OID_ASSOCIATEEX)) {
 			ieofs = 4;
 		} else if (hdr->oid == htonl(DOT11_OID_REASSOCIATEEX)) {
@@ -802,7 +802,7 @@ static void prism54_handle_assoc(struct prism54_driver_data *drv,
 			return;
 		}
 		memcpy(&mlme->address[0], mac_id_get(drv, mlme->id), ETH_ALEN);
-		sta = ap_get_sta(drv->hapd, &mlme->address[0]);
+		sta = ap_get_sta(drv->hapd, (u8 *) &mlme->address[0]);
 		if (sta == NULL) {
 			printf("cannot get sta\n");
 			return;
@@ -845,7 +845,7 @@ static void handle_pim(int sock, void *eloop_ctx, void *sock_ctx)
 		return;
 	}
 
-	if (hdr->op != htonl(PIMOP_TRAP)) {
+	if (hdr->op != (int) htonl(PIMOP_TRAP)) {
 		free(hdr);
 		return;
 	}
