@@ -689,7 +689,7 @@ static u8 * eap_sm_buildFailure(struct eap_sm *sm, int id, size_t *len)
 static int eap_sm_nextId(struct eap_sm *sm, int id)
 {
 	if (id < 0) {
-		/* RFC 3748 Ch 4.1: recommended to initalize Identifier with a
+		/* RFC 3748 Ch 4.1: recommended to initialize Identifier with a
 		 * random number */
 		id = rand() & 0xff;
 		if (id != sm->lastId)
@@ -710,7 +710,8 @@ static int eap_sm_nextId(struct eap_sm *sm, int id)
  */
 void eap_sm_process_nak(struct eap_sm *sm, u8 *nak_list, size_t len)
 {
-	int i, j;
+	int i;
+	size_t j;
 
 	wpa_printf(MSG_MSGDUMP, "EAP: processing NAK (current EAP method "
 		   "index %d)", sm->user_eap_method_index);
