@@ -164,7 +164,7 @@ static int eap_aka_build_encr(struct eap_sm *sm, struct eap_aka_data *data,
 			   data->next_pseudonym);
 		eap_sim_msg_add(msg, EAP_SIM_AT_NEXT_PSEUDONYM,
 				strlen(data->next_pseudonym),
-				data->next_pseudonym,
+				(u8 *) data->next_pseudonym,
 				strlen(data->next_pseudonym));
 	}
 
@@ -173,7 +173,7 @@ static int eap_aka_build_encr(struct eap_sm *sm, struct eap_aka_data *data,
 			   data->next_reauth_id);
 		eap_sim_msg_add(msg, EAP_SIM_AT_NEXT_REAUTH_ID,
 				strlen(data->next_reauth_id),
-				data->next_reauth_id,
+				(u8 *) data->next_reauth_id,
 				strlen(data->next_reauth_id));
 	}
 
