@@ -1,6 +1,6 @@
 /*
  * hostapd / EAP-GTC (RFC 3748)
- * Copyright (c) 2004, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2004-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -28,10 +28,9 @@ static void * eap_gtc_init(struct eap_sm *sm)
 {
 	struct eap_gtc_data *data;
 
-	data = malloc(sizeof(*data));
+	data = wpa_zalloc(sizeof(*data));
 	if (data == NULL)
-		return data;
-	memset(data, 0, sizeof(*data));
+		return NULL;
 	data->state = CONTINUE;
 
 	return data;
@@ -55,8 +54,8 @@ static u8 * eap_gtc_buildReq(struct eap_sm *sm, void *priv, int id,
 	size_t msg_len;
 
 	msg_len = strlen(msg);
-	*reqDataLen = sizeof(*req) + 1 + msg_len;
-	req = malloc(*reqDataLen);
+	req = eap_msg_alloc(EAP_VENDOR_IETF, EAP_TYPE_GTC, reqDataLen,
+			    msg_len, EAP_CODE_REQUEST, id, &pos);
 	if (req == NULL) {
 		wpa_printf(MSG_ERROR, "EAP-GTC: Failed to allocate memory for "
 			   "request");
@@ -64,11 +63,6 @@ static u8 * eap_gtc_buildReq(struct eap_sm *sm, void *priv, int id,
 		return NULL;
 	}
 
-	req->code = EAP_CODE_REQUEST;
-	req->identifier = id;
-	req->length = htons(*reqDataLen);
-	pos = (u8 *) (req + 1);
-	*pos++ = EAP_TYPE_GTC;
 	memcpy(pos, msg, msg_len);
 
 	data->state = CONTINUE;
@@ -80,14 +74,12 @@ static u8 * eap_gtc_buildReq(struct eap_sm *sm, void *priv, int id,
 static Boolean eap_gtc_check(struct eap_sm *sm, void *priv,
 			     u8 *respData, size_t respDataLen)
 {
-	struct eap_hdr *resp;
-	u8 *pos;
+	const u8 *pos;
 	size_t len;
 
-	resp = (struct eap_hdr *) respData;
-	pos = (u8 *) (resp + 1);
-	if (respDataLen < sizeof(*resp) + 2 || *pos != EAP_TYPE_GTC ||
-	    (len = ntohs(resp->length)) > respDataLen) {
+	pos = eap_hdr_validate(EAP_VENDOR_IETF, EAP_TYPE_GTC,
+			       respData, respDataLen, &len);
+	if (pos == NULL || len < 1) {
 		wpa_printf(MSG_INFO, "EAP-GTC: Invalid frame");
 		return TRUE;
 	}
@@ -100,8 +92,7 @@ static void eap_gtc_process(struct eap_sm *sm, void *priv,
 			    u8 *respData, size_t respDataLen)
 {
 	struct eap_gtc_data *data = priv;
-	struct eap_hdr *resp;
-	u8 *pos;
+	const u8 *pos;
 	size_t rlen;
 
 	if (sm->user == NULL || sm->user->password == NULL ||
@@ -112,10 +103,11 @@ static void eap_gtc_process(struct eap_sm *sm, void *priv,
 		return;
 	}
 
-	resp = (struct eap_hdr *) respData;
-	pos = (u8 *) (resp + 1);
-	pos++;
-	rlen = ntohs(resp->length) - sizeof(*resp) - 1;
+	pos = eap_hdr_validate(EAP_VENDOR_IETF, EAP_TYPE_GTC,
+			       respData, respDataLen, &rlen);
+	if (pos == NULL || rlen < 1)
+		return; /* Should not happen - frame already validated */
+
 	wpa_hexdump_key(MSG_MSGDUMP, "EAP-GTC: Response", pos, rlen);
 
 	if (rlen != sm->user->password_len ||
