@@ -170,5 +170,10 @@ struct eap_sm {
 int eap_user_get(struct eap_sm *sm, const u8 *identity, size_t identity_len,
 		 int phase2);
 void eap_sm_process_nak(struct eap_sm *sm, u8 *nak_list, size_t len);
+const u8 * eap_hdr_validate(int vendor, EapType eap_type,
+			    const u8 *msg, size_t msglen, size_t *plen);
+struct eap_hdr * eap_msg_alloc(int vendor, EapType type, size_t *len,
+			       size_t payload_len, u8 code, u8 identifier,
+			       u8 **payload);
 
 #endif /* EAP_I_H */
