@@ -16,6 +16,7 @@
 
 #include "common.h"
 #include "aes_wrap.h"
+#include "eap_defs.h"
 #include "eap_psk_common.h"
 
 #define aes_block_size 16
@@ -46,7 +47,7 @@ void eap_psk_derive_keys(const u8 *kdk, const u8 *rand_p, u8 *tek, u8 *msk)
 	hash[aes_block_size - 1] ^= counter;
 	counter++;
 
-	for (i = 0; i < EAP_PSK_MSK_LEN / aes_block_size; i++) {
+	for (i = 0; i < EAP_MSK_LEN / aes_block_size; i++) {
 		hash[aes_block_size - 1] ^= counter;
 		aes_128_encrypt_block(kdk, hash, &msk[i * aes_block_size]);
 		hash[aes_block_size - 1] ^= counter;
