@@ -288,7 +288,7 @@ static void eap_sim_db_receive(int sock, void *eloop_ctx, void *sock_ctx)
 			      "external source", (u8 *) buf, res);
 	if (res == 0)
 		return;
-	if (res >= sizeof(buf))
+	if (res >= (int) sizeof(buf))
 		res = sizeof(buf) - 1;
 	buf[res] = '\0';
 
@@ -399,11 +399,10 @@ void * eap_sim_db_init(const char *config,
 {
 	struct eap_sim_db_data *data;
 
-	data = malloc(sizeof(*data));
+	data = wpa_zalloc(sizeof(*data));
 	if (data == NULL)
 		return NULL;
 
-	memset(data, 0, sizeof(*data));
 	data->sock = -1;
 	data->get_complete_cb = get_complete_cb;
 	data->ctx = ctx;
@@ -551,7 +550,8 @@ int eap_sim_db_get_gsm_triplets(void *priv, const u8 *identity,
 {
 	struct eap_sim_db_data *data = priv;
 	struct eap_sim_db_pending *entry;
-	int i, len;
+	int len, ret;
+	size_t i;
 	char msg[40];
 
 	if (identity_len < 2 || identity[0] != EAP_SIM_PERMANENT_PREFIX ||
@@ -606,22 +606,24 @@ int eap_sim_db_get_gsm_triplets(void *priv, const u8 *identity,
 	}
 
 	len = snprintf(msg, sizeof(msg), "SIM-REQ-AUTH ");
-	if (len + identity_len >= sizeof(msg))
+	if (len < 0 || len + identity_len >= sizeof(msg))
 		return EAP_SIM_DB_FAILURE;
 	memcpy(msg + len, identity, identity_len);
 	len += identity_len;
-	len += snprintf(msg + len, sizeof(msg) - len, " %d", max_chal);
+	ret = snprintf(msg + len, sizeof(msg) - len, " %d", max_chal);
+	if (ret < 0 || (size_t) ret >= sizeof(msg) - len)
+		return EAP_SIM_DB_FAILURE;
+	len += ret;
 
 	wpa_hexdump(MSG_DEBUG, "EAP-SIM DB: requesting SIM authentication "
 		    "data for IMSI", identity, identity_len);
 	if (eap_sim_db_send(data, msg, len) < 0)
 		return EAP_SIM_DB_FAILURE;
 
-	entry = malloc(sizeof(*entry));
+	entry = wpa_zalloc(sizeof(*entry));
 	if (entry == NULL)
 		return EAP_SIM_DB_FAILURE;
 
-	memset(entry, 0, sizeof(*entry));
 	os_get_time(&entry->timestamp);
 	memcpy(entry->imsi, identity, identity_len);
 	entry->imsi_len = identity_len;
@@ -911,13 +913,12 @@ int eap_sim_db_add_pseudonym(void *priv, const u8 *identity,
 		return 0;
 	}
 
-	p = malloc(sizeof(*p));
+	p = wpa_zalloc(sizeof(*p));
 	if (p == NULL) {
 		free(pseudonym);
 		return -1;
 	}
 
-	memset(p, 0, sizeof(*p));
 	p->next = data->pseudonyms;
 	p->identity = malloc(identity_len);
 	if (p->identity == NULL) {
@@ -970,13 +971,12 @@ int eap_sim_db_add_reauth(void *priv, const u8 *identity,
 		free(r->reauth_id);
 		r->reauth_id = reauth_id;
 	} else {
-		r = malloc(sizeof(*r));
+		r = wpa_zalloc(sizeof(*r));
 		if (r == NULL) {
 			free(reauth_id);
 			return -1;
 		}
 
-		memset(r, 0, sizeof(*r));
 		r->next = data->reauths;
 		r->identity = malloc(identity_len);
 		if (r->identity == NULL) {
@@ -1107,7 +1107,8 @@ int eap_sim_db_get_aka_auth(void *priv, const u8 *identity,
 {
 	struct eap_sim_db_data *data = priv;
 	struct eap_sim_db_pending *entry;
-	int i, len;
+	int len;
+	size_t i;
 	char msg[40];
 
 	if (identity_len < 2 || identity[0] != EAP_AKA_PERMANENT_PREFIX ||
@@ -1159,7 +1160,7 @@ int eap_sim_db_get_aka_auth(void *priv, const u8 *identity,
 	}
 
 	len = snprintf(msg, sizeof(msg), "AKA-REQ-AUTH ");
-	if (len + identity_len >= sizeof(msg))
+	if (len < 0 || len + identity_len >= sizeof(msg))
 		return EAP_SIM_DB_FAILURE;
 	memcpy(msg + len, identity, identity_len);
 	len += identity_len;
@@ -1169,11 +1170,10 @@ int eap_sim_db_get_aka_auth(void *priv, const u8 *identity,
 	if (eap_sim_db_send(data, msg, len) < 0)
 		return EAP_SIM_DB_FAILURE;
 
-	entry = malloc(sizeof(*entry));
+	entry = wpa_zalloc(sizeof(*entry));
 	if (entry == NULL)
 		return EAP_SIM_DB_FAILURE;
 
-	memset(entry, 0, sizeof(*entry));
 	os_get_time(&entry->timestamp);
 	entry->aka = 1;
 	memcpy(entry->imsi, identity, identity_len);
@@ -1217,18 +1217,24 @@ int eap_sim_db_resynchronize(void *priv, const u8 *identity,
 
 	if (data->sock >= 0) {
 		char msg[100];
-		int len;
+		int len, ret;
 
 		len = snprintf(msg, sizeof(msg), "AKA-AUTS ");
-		if (len + identity_len - 1 >= sizeof(msg))
+		if (len < 0 || len + identity_len - 1 >= sizeof(msg))
 			return -1;
 		memcpy(msg + len, identity + 1, identity_len - 1);
 		len += identity_len - 1;
 
-		len += snprintf(msg + len, sizeof(msg) - len, " ");
+		ret = snprintf(msg + len, sizeof(msg) - len, " ");
+		if (ret < 0 || (size_t) ret >= sizeof(msg) - len)
+			return -1;
+		len += ret;
 		len += wpa_snprintf_hex(msg + len, sizeof(msg) - len,
 					auts, EAP_AKA_AUTS_LEN);
-		len += snprintf(msg + len, sizeof(msg) - len, " ");
+		ret = snprintf(msg + len, sizeof(msg) - len, " ");
+		if (ret < 0 || (size_t) ret >= sizeof(msg) - len)
+			return -1;
+		len += ret;
 		len += wpa_snprintf_hex(msg + len, sizeof(msg) - len,
 					rand, EAP_AKA_RAND_LEN);
 		wpa_hexdump(MSG_DEBUG, "EAP-SIM DB: reporting AKA AUTS for "
