@@ -1,3 +1,17 @@
+/*
+ * hostapd / EAP-SIM database/authenticator gateway
+ * Copyright (c) 2005-2006, Jouni Malinen <jkmaline@cc.hut.fi>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
+ *
+ * Alternatively, this software may be distributed under the terms of BSD
+ * license.
+ *
+ * See README and COPYING for more details.
+ */
+
 #ifndef EAP_SIM_DB_H
 #define EAP_SIM_DB_H
 
@@ -65,7 +79,8 @@ int eap_sim_db_get_aka_auth(void *priv, const u8 *identity,
 			    void *cb_session_ctx);
 
 int eap_sim_db_resynchronize(void *priv, const u8 *identity,
-			     size_t identity_len, const u8 *auts);
+			     size_t identity_len, const u8 *auts,
+			     const u8 *rand);
 
 #else /* EAP_SIM */
 static inline void *
