@@ -183,7 +183,7 @@ static int eap_ttls_avp_parse(u8 *buf, size_t len, struct eap_ttls_avp *parse)
 		wpa_printf(MSG_DEBUG, "EAP-TTLS: AVP: code=%d flags=0x%02x "
 			   "length=%d", (int) avp_code, avp_flags,
 			   (int) avp_length);
-		if (avp_length > left) {
+		if ((int) avp_length > left) {
 			wpa_printf(MSG_WARNING, "EAP-TTLS: AVP overflow "
 				   "(len=%d, left=%d) - dropped",
 				   (int) avp_length, left);
@@ -365,10 +365,9 @@ static void * eap_ttls_init(struct eap_sm *sm)
 {
 	struct eap_ttls_data *data;
 
-	data = malloc(sizeof(*data));
+	data = wpa_zalloc(sizeof(*data));
 	if (data == NULL)
-		return data;
-	memset(data, 0, sizeof(*data));
+		return NULL;
 	data->ttls_version = EAP_TTLS_VERSION;
 	data->force_version = -1;
 	if (sm->user && sm->user->force_version >= 0) {
@@ -539,6 +538,7 @@ static u8 * eap_ttls_build_phase2_mschapv2(struct eap_sm *sm,
 					   int id, size_t *reqDataLen)
 {
 	u8 *req, *encr_req, *pos, *end;
+	int ret;
 	size_t req_len;
 
 	pos = req = malloc(100);
@@ -550,7 +550,9 @@ static u8 * eap_ttls_build_phase2_mschapv2(struct eap_sm *sm,
 		pos = eap_ttls_avp_hdr(pos, RADIUS_ATTR_MS_CHAP2_SUCCESS,
 				       RADIUS_VENDOR_ID_MICROSOFT, 1, 43);
 		*pos++ = data->mschapv2_ident;
-		pos += snprintf((char *) pos, end - pos, "S=");
+		ret = snprintf((char *) pos, end - pos, "S=");
+		if (ret >= 0 && ret < end - pos)
+			pos += ret;
 		pos += wpa_snprintf_hex_uppercase(
 			(char *) pos, end - pos, data->mschapv2_auth_response,
 			sizeof(data->mschapv2_auth_response));
@@ -857,8 +859,7 @@ static void eap_ttls_process_phase2_mschapv2(struct eap_sm *sm,
 {
 	u8 *chal, *username, nt_response[24], *pos, *rx_resp, *peer_challenge,
 		*auth_challenge;
-	size_t username_len;
-	int i;
+	size_t username_len, i;
 
 	if (challenge == NULL || response == NULL ||
 	    challenge_len != EAP_TTLS_MSCHAPV2_CHALLENGE_LEN ||
@@ -1164,8 +1165,9 @@ static void eap_ttls_process_phase2(struct eap_sm *sm,
 				    u8 *in_data, size_t in_len)
 {
 	u8 *in_decrypted;
-	int buf_len, len_decrypted, res;
+	int len_decrypted, res;
 	struct eap_ttls_avp parse;
+	size_t buf_len;
 
 	wpa_printf(MSG_DEBUG, "EAP-TTLS: received %lu bytes encrypted data for"
 		   " Phase 2", (unsigned long) in_len);
