@@ -663,12 +663,11 @@ eapol_sm_alloc(struct hostapd_data *hapd, struct sta_info *sta)
 {
 	struct eapol_state_machine *sm;
 
-	sm = (struct eapol_state_machine *) malloc(sizeof(*sm));
+	sm = wpa_zalloc(sizeof(*sm));
 	if (sm == NULL) {
 		printf("IEEE 802.1X port state allocation failed\n");
 		return NULL;
 	}
-	memset(sm, 0, sizeof(*sm));
 	sm->radius_identifier = -1;
 	memcpy(sm->addr, sta->addr, ETH_ALEN);
 	if (sta->flags & WLAN_STA_PREAUTH)
@@ -754,8 +753,8 @@ static void eapol_sm_step_run(struct eapol_state_machine *sm)
 {
 	struct hostapd_data *hapd = sm->hapd;
 	u8 addr[ETH_ALEN];
-	int prev_auth_pae, prev_be_auth, prev_reauth_timer, prev_auth_key_tx,
-		prev_key_rx, prev_ctrl_dir;
+	unsigned int prev_auth_pae, prev_be_auth, prev_reauth_timer,
+		prev_auth_key_tx, prev_key_rx, prev_ctrl_dir;
 	int max_steps = 100;
 
 	memcpy(addr, sm->sta->addr, ETH_ALEN);
