@@ -1,3 +1,17 @@
+/*
+ * hostapd / IEEE 802.1X Authenticator - EAPOL state machine
+ * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
+ *
+ * Alternatively, this software may be distributed under the terms of BSD
+ * license.
+ *
+ * See README and COPYING for more details.
+ */
+
 #ifndef EAPOL_SM_H
 #define EAPOL_SM_H
 
@@ -181,7 +195,7 @@ struct eapol_state_machine {
 };
 
 
-struct eapol_state_machine *eapol_sm_alloc(hostapd *hapd,
+struct eapol_state_machine *eapol_sm_alloc(struct hostapd_data *hapd,
 					   struct sta_info *sta);
 void eapol_sm_free(struct eapol_state_machine *sm);
 void eapol_sm_step(struct eapol_state_machine *sm);
