@@ -51,6 +51,7 @@
 #define WLAN_FC_STYPE_DISASSOC 10
 #define WLAN_FC_STYPE_AUTH 11
 #define WLAN_FC_STYPE_DEAUTH 12
+#define WLAN_FC_STYPE_ACTION 13
 
 /* control */
 #define WLAN_FC_STYPE_PSPOLL 10
@@ -81,6 +82,12 @@
 #define WLAN_CAPABILITY_CF_POLLABLE BIT(2)
 #define WLAN_CAPABILITY_CF_POLL_REQUEST BIT(3)
 #define WLAN_CAPABILITY_PRIVACY BIT(4)
+#define WLAN_CAPABILITY_SHORT_PREAMBLE BIT(5)
+#define WLAN_CAPABILITY_PBCC BIT(6)
+#define WLAN_CAPABILITY_CHANNEL_AGILITY BIT(7)
+#define WLAN_CAPABILITY_SPECTRUM_MGMT BIT(8)
+#define WLAN_CAPABILITY_SHORT_SLOT_TIME BIT(10)
+#define WLAN_CAPABILITY_DSSS_OFDM BIT(13)
 
 /* Status codes */
 #define WLAN_STATUS_SUCCESS 0
@@ -141,9 +148,11 @@
 #define WLAN_EID_TIM 5
 #define WLAN_EID_IBSS_PARAMS 6
 #define WLAN_EID_CHALLENGE 16
+#define WLAN_EID_ERP_INFO 42
 #define WLAN_EID_RSN 48
 #define WLAN_EID_EXT_SUPP_RATES 50
 #define WLAN_EID_GENERIC 221
+#define WLAN_EID_VENDOR_SPECIFIC 221
 
 
 /* HFA384X Configuration RIDs */
