@@ -28,12 +28,14 @@
 #include "iapp.h"
 #include "ap.h"
 #include "ieee802_11_auth.h"
+#include "ap_list.h"
 #include "sta_info.h"
 #include "driver.h"
 #include "radius_client.h"
 #include "radius_server.h"
 #include "wpa.h"
 #include "preauth.h"
+#include "wme.h"
 #include "ctrl_iface.h"
 #include "tls.h"
 #include "eap_sim_db.h"
@@ -220,6 +222,8 @@ void hostapd_new_assoc_sta(struct hostapd_data *hapd, struct sta_info *sta,
 	if (!hapd->conf->ieee802_1x && !hapd->conf->wpa)
 		accounting_sta_start(hapd, sta);
 
+	hostapd_wme_sta_config(hapd, sta);
+
 	/* Start IEEE 802.1X authentication process for new stations */
 	ieee802_1x_new_station(hapd, sta);
 	if (reassoc)
@@ -268,6 +272,8 @@ static void hostapd_wpa_auth_conf(struct hostapd_bss_config *conf,
 	wconf->rsn_preauth = conf->rsn_preauth;
 	wconf->stakey = conf->stakey;
 	wconf->eapol_version = conf->eapol_version;
+	wconf->peerkey = conf->peerkey;
+	wconf->wme_enabled = conf->wme_enabled;
 }
 
 
@@ -329,12 +335,18 @@ static void hostapd_dump_state(struct hostapd_data *hapd)
 
 	time(&now);
 	fprintf(f, "hostapd state dump - %s", ctime(&now));
+	fprintf(f, "num_sta=%d num_sta_non_erp=%d "
+		"num_sta_no_short_slot_time=%d\n"
+		"num_sta_no_short_preamble=%d\n",
+		hapd->num_sta, hapd->iface->num_sta_non_erp,
+		hapd->iface->num_sta_no_short_slot_time,
+		hapd->iface->num_sta_no_short_preamble);
 
 	for (sta = hapd->sta_list; sta != NULL; sta = sta->next) {
 		fprintf(f, "\nSTA=" MACSTR "\n", MAC2STR(sta->addr));
 
 		fprintf(f,
-			"  AID=%d flags=0x%x %s%s%s%s%s%s\n"
+			"  AID=%d flags=0x%x %s%s%s%s%s%s%s%s%s%s\n"
 			"  capability=0x%x listen_interval=%d\n",
 			sta->aid,
 			sta->flags,
@@ -345,6 +357,12 @@ static void hostapd_dump_state(struct hostapd_data *hapd)
 			(sta->flags & WLAN_STA_PERM ? "[PERM]" : ""),
 			(sta->flags & WLAN_STA_AUTHORIZED ? "[AUTHORIZED]" :
 			 ""),
+			(sta->flags & WLAN_STA_PENDING_POLL ? "[PENDING_POLL" :
+			 ""),
+			(sta->flags & WLAN_STA_SHORT_PREAMBLE ?
+			 "[SHORT_PREAMBLE]" : ""),
+			(sta->flags & WLAN_STA_PREAUTH ? "[PREAUTH]" : ""),
+			(sta->flags & WLAN_STA_NONERP ? "[NonERP]" : ""),
 			sta->capability,
 			sta->listen_interval);
 
@@ -455,6 +473,7 @@ static void hostapd_cleanup_iface(struct hostapd_iface *iface)
 	hostapd_free_hw_features(iface->hw_features, iface->num_hw_features);
 	free(iface->current_rates);
 	hostapd_config_free(iface->conf);
+	ap_list_deinit(iface);
 
 	free(iface->config_fname);
 	free(iface->bss);
@@ -692,7 +711,7 @@ static int hostapd_setup_interface(struct hostapd_iface *iface)
 		 * from what is being used then force installation of the
 		 * new SSID.
 		 */
-		set_ssid = (conf->ssid.ssid_len != ssid_len ||
+		set_ssid = (conf->ssid.ssid_len != (size_t) ssid_len ||
 			    memcmp(conf->ssid.ssid, ssid, ssid_len) != 0);
 	} else {
 		/*
@@ -813,7 +832,12 @@ static int hostapd_setup_interface(struct hostapd_iface *iface)
 	if (hostapd_wireless_event_init(hapd) < 0)
 		return -1;
 
-	ieee802_11_set_beacon(hapd);
+	if (hostapd_get_hw_features(iface)) {
+		/* Not all drivers support this yet, so continue without hw
+		 * feature data. */
+	} else if (hostapd_select_hw_mode(iface)) {
+		printf("Could not select hw_mode and channel.\n");
+	}
 
 	if (hostapd_flush_old_stations(hapd))
 		return -1;
@@ -823,13 +847,6 @@ static int hostapd_setup_interface(struct hostapd_iface *iface)
 		ret = -1;
 	}
 
-	if (hostapd_get_hw_features(iface)) {
-		/* Not all drivers support this yet, so continue without hw
-		 * feature data. */
-	} else if (hostapd_select_hw_mode(iface)) {
-		printf("Could not select hw_mode and channel.\n");
-	}
-
 	if (hapd->iconf->channel) {
 		freq = hostapd_hw_get_freq(hapd, hapd->iconf->channel);
 		printf("Mode: %s  Channel: %d  Frequency: %d MHz\n",
@@ -842,6 +859,11 @@ static int hostapd_setup_interface(struct hostapd_iface *iface)
 		}
 	}
 
+	hostapd_set_beacon_int(hapd, hapd->iconf->beacon_int);
+	ieee802_11_set_beacon(hapd);
+
+	ap_list_init(iface);
+
 	return ret;
 }
 
@@ -941,12 +963,14 @@ static void usage(void)
 	show_version();
 	fprintf(stderr,
 		"\n"
-		"usage: hostapd [-hdBKt] <configuration file(s)>\n"
+		"usage: hostapd [-hdBKtv] [-P <PID file>] "
+		"<configuration file(s)>\n"
 		"\n"
 		"options:\n"
 		"   -h   show this usage\n"
 		"   -d   show more debug messages (-dd for even more)\n"
 		"   -B   run daemon in the background\n"
+		"   -P   PID file\n"
 		"   -K   include key data in debug messages\n"
 		"   -t   include timestamps in some debug messages\n"
 		"   -v   show hostapd version\n");
@@ -1061,7 +1085,7 @@ static struct hostapd_iface * hostapd_init(const char *config_file)
 	if (hapd_iface->bss == NULL)
 		goto fail;
 
-	for (i = 0; i < conf->num_bss; i++) {
+	for (i = 0; (size_t) i < conf->num_bss; i++) {
 		hapd = hapd_iface->bss[i] =
 			hostapd_alloc_bss_data(hapd_iface, conf,
 					       &conf->bss[i]);
@@ -1101,9 +1125,10 @@ int main(int argc, char *argv[])
 	struct hapd_interfaces interfaces;
 	int ret = 1, i, j;
 	int c, debug = 0, daemonize = 0;
+	const char *pid_file = NULL;
 
 	for (;;) {
-		c = getopt(argc, argv, "BdhKtv");
+		c = getopt(argc, argv, "BdhKP:tv");
 		if (c < 0)
 			break;
 		switch (c) {
@@ -1121,6 +1146,9 @@ int main(int argc, char *argv[])
 		case 'K':
 			wpa_debug_show_keys++;
 			break;
+		case 'P':
+			pid_file = optarg;
+			break;
 		case 't':
 			wpa_debug_timestamp++;
 			break;
@@ -1181,7 +1209,7 @@ int main(int argc, char *argv[])
 			goto out;
 	}
 
-	if (daemonize && os_daemonize(NULL)) {
+	if (daemonize && os_daemonize(pid_file)) {
 		perror("daemon");
 		goto out;
 	}
@@ -1230,7 +1258,7 @@ int main(int argc, char *argv[])
 
 	driver_unregister_all();
 
-	os_daemonize_terminate(NULL);
+	os_daemonize_terminate(pid_file);
 
 	return ret;
 }
