@@ -31,6 +31,8 @@
 #include "rc4.h"
 #include "ieee802_1x.h"
 #include "wpa.h"
+#include "wme.h"
+#include "ap_list.h"
 #include "accounting.h"
 #include "driver.h"
 #include "hostap_common.h"
@@ -104,6 +106,10 @@ u16 hostapd_own_capab_info(struct hostapd_data *hapd, struct sta_info *sta,
 	int capab = WLAN_CAPABILITY_ESS;
 	int privacy;
 
+	if (hapd->iface->num_sta_no_short_preamble == 0 &&
+	    hapd->iconf->preamble == SHORT_PREAMBLE)
+		capab |= WLAN_CAPABILITY_SHORT_PREAMBLE;
+
 	privacy = hapd->conf->ssid.wep.keys_set;
 
 	if (hapd->conf->ieee802_1x &&
@@ -131,11 +137,96 @@ u16 hostapd_own_capab_info(struct hostapd_data *hapd, struct sta_info *sta,
 	if (privacy)
 		capab |= WLAN_CAPABILITY_PRIVACY;
 
+	if (hapd->iface && hapd->iface->current_mode &&
+	    hapd->iface->current_mode->mode == HOSTAPD_MODE_IEEE80211G &&
+	    hapd->iface->num_sta_no_short_slot_time == 0)
+		capab |= WLAN_CAPABILITY_SHORT_SLOT_TIME;
+
 	return capab;
 }
 
 
-static const u8 WPA_OUI_TYPE[] = { 0x00, 0x50, 0xf2, 1 };
+#define OUI_MICROSOFT 0x0050f2 /* Microsoft (also used in Wi-Fi specs)
+				* 00:50:F2 */
+
+static int ieee802_11_parse_vendor_specific(struct hostapd_data *hapd,
+					    u8 *pos, size_t elen,
+					    struct ieee802_11_elems *elems,
+					    int show_errors)
+{
+	unsigned int oui;
+
+	/* first 3 bytes in vendor specific information element are the IEEE
+	 * OUI of the vendor. The following byte is used a vendor specific
+	 * sub-type. */
+	if (elen < 4) {
+		if (show_errors) {
+			HOSTAPD_DEBUG(HOSTAPD_DEBUG_MSGDUMPS, "short vendor "
+				      "specific information element ignored "
+				      "(len=%lu)\n", (unsigned long) elen);
+		}
+		return -1;
+	}
+
+	oui = (pos[0] << 16) | (pos[1] << 8) | pos[2];
+	switch (oui) {
+	case OUI_MICROSOFT:
+		/* Microsoft/Wi-Fi information elements are further typed and
+		 * subtyped */
+		switch (pos[3]) {
+		case 1:
+			/* Microsoft OUI (00:50:F2) with OUI Type 1:
+			 * real WPA information element */
+			elems->wpa_ie = pos;
+			elems->wpa_ie_len = elen;
+			break;
+		case WME_OUI_TYPE: /* this is a Wi-Fi WME info. element */
+			if (elen < 5) {
+				HOSTAPD_DEBUG(HOSTAPD_DEBUG_MSGDUMPS,
+					      "short WME information element "
+					      "ignored (len=%lu)\n",
+					      (unsigned long) elen);
+				return -1;
+			}
+			switch (pos[4]) {
+			case WME_OUI_SUBTYPE_INFORMATION_ELEMENT:
+			case WME_OUI_SUBTYPE_PARAMETER_ELEMENT:
+				elems->wme = pos;
+				elems->wme_len = elen;
+				break;
+			case WME_OUI_SUBTYPE_TSPEC_ELEMENT:
+				elems->wme_tspec = pos;
+				elems->wme_tspec_len = elen;
+				break;
+			default:
+				HOSTAPD_DEBUG(HOSTAPD_DEBUG_MSGDUMPS,
+					      "unknown WME information element"
+					      " ignored (subtype=%d "
+					      "len=%lu)\n",
+					      pos[4], (unsigned long) elen);
+				return -1;
+			}
+			break;
+		default:
+			HOSTAPD_DEBUG(HOSTAPD_DEBUG_MSGDUMPS,
+				      "Unknown Microsoft information element "
+				      "ignored (type=%d len=%lu)\n",
+				      pos[3], (unsigned long) elen);
+			return -1;
+		}
+		break;
+
+	default:
+		HOSTAPD_DEBUG(HOSTAPD_DEBUG_MSGDUMPS,
+			      "unknown vendor specific information element "
+			      "ignored (vendor OUI %02x:%02x:%02x len=%lu)\n",
+			      pos[0], pos[1], pos[2], (unsigned long) elen);
+		return -1;
+	}
+
+	return 0;
+}
+
 
 ParseRes ieee802_11_parse_elems(struct hostapd_data *hapd, u8 *start,
 				size_t len,
@@ -200,29 +291,19 @@ ParseRes ieee802_11_parse_elems(struct hostapd_data *hapd, u8 *start,
 			elems->challenge = pos;
 			elems->challenge_len = elen;
 			break;
+		case WLAN_EID_ERP_INFO:
+			elems->erp_info = pos;
+			elems->erp_info_len = elen;
+			break;
 		case WLAN_EID_EXT_SUPP_RATES:
 			elems->ext_supp_rates = pos;
 			elems->ext_supp_rates_len = elen;
 			break;
-		case WLAN_EID_GENERIC:
-			if (elen > 4 && memcmp(pos, WPA_OUI_TYPE, 4) == 0) {
-				elems->wpa_ie = pos;
-				elems->wpa_ie_len = elen;
-			} else if (show_errors) {
-				HOSTAPD_DEBUG(HOSTAPD_DEBUG_EXCESSIVE,
-					      "IEEE 802.11 element parse "
-					      "ignored unknown generic element"
-					      " (id=%d elen=%d OUI:type="
-					      "%02x-%02x-%02x:%d)\n",
-					      id, elen,
-					      elen >= 1 ? pos[0] : 0,
-					      elen >= 2 ? pos[1] : 0,
-					      elen >= 3 ? pos[2] : 0,
-					      elen >= 4 ? pos[3] : 0);
+		case WLAN_EID_VENDOR_SPECIFIC:
+			if (ieee802_11_parse_vendor_specific(hapd, pos, elen,
+							     elems,
+							     show_errors))
 				unknown++;
-			} else {
-				unknown++;
-			}
 			break;
 		case WLAN_EID_RSN:
 			elems->rsn_ie = pos;
@@ -602,7 +683,7 @@ static void handle_assoc(struct hostapd_data *hapd,
 	u16 resp = WLAN_STATUS_SUCCESS;
 	u8 *pos, *wpa_ie;
 	size_t wpa_ie_len;
-	int send_deauth = 0, send_len, left;
+	int send_deauth = 0, send_len, left, i;
 	struct sta_info *sta;
 	struct ieee802_11_elems elems;
 
@@ -676,6 +757,18 @@ static void handle_assoc(struct hostapd_data *hapd,
 		goto fail;
 	}
 
+	sta->flags &= ~WLAN_STA_WME;
+	if (elems.wme && hapd->conf->wme_enabled) {
+		if (hostapd_eid_wme_valid(hapd, elems.wme, elems.wme_len))
+			hostapd_logger(hapd, sta->addr,
+				       HOSTAPD_MODULE_WPA,
+				       HOSTAPD_LEVEL_DEBUG,
+				       "invalid WME element in association "
+				       "request");
+		else
+			sta->flags |= WLAN_STA_WME;
+	}
+
 	if (!elems.supp_rates) {
 		hostapd_logger(hapd, mgmt->sa, HOSTAPD_MODULE_IEEE80211,
 			       HOSTAPD_LEVEL_DEBUG,
@@ -761,6 +854,45 @@ static void handle_assoc(struct hostapd_data *hapd,
 			goto fail;
 	}
 
+	if (hapd->iface->current_mode->mode == HOSTAPD_MODE_IEEE80211G)
+		sta->flags |= WLAN_STA_NONERP;
+	for (i = 0; i < sta->supported_rates_len; i++) {
+		if ((sta->supported_rates[i] & 0x7f) > 22) {
+			sta->flags &= ~WLAN_STA_NONERP;
+			break;
+		}
+	}
+	if (sta->flags & WLAN_STA_NONERP && !sta->nonerp_set) {
+		sta->nonerp_set = 1;
+		hapd->iface->num_sta_non_erp++;
+		if (hapd->iface->num_sta_non_erp == 1)
+			ieee802_11_set_beacons(hapd->iface);
+	}
+
+	if (!(sta->capability & WLAN_CAPABILITY_SHORT_SLOT_TIME) &&
+	    !sta->no_short_slot_time_set) {
+		sta->no_short_slot_time_set = 1;
+		hapd->iface->num_sta_no_short_slot_time++;
+		if (hapd->iface->current_mode->mode ==
+		    HOSTAPD_MODE_IEEE80211G &&
+		    hapd->iface->num_sta_no_short_slot_time == 1)
+			ieee802_11_set_beacons(hapd->iface);
+	}
+
+	if (sta->capability & WLAN_CAPABILITY_SHORT_PREAMBLE)
+		sta->flags |= WLAN_STA_SHORT_PREAMBLE;
+	else
+		sta->flags &= ~WLAN_STA_SHORT_PREAMBLE;
+
+	if (!(sta->capability & WLAN_CAPABILITY_SHORT_PREAMBLE) &&
+	    !sta->no_short_preamble_set) {
+		sta->no_short_preamble_set = 1;
+		hapd->iface->num_sta_no_short_preamble++;
+		if (hapd->iface->current_mode->mode == HOSTAPD_MODE_IEEE80211G
+		    && hapd->iface->num_sta_no_short_preamble == 1)
+			ieee802_11_set_beacons(hapd->iface);
+	}
+
 	/* get a unique AID */
 	if (sta->aid > 0) {
 		HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
@@ -826,6 +958,8 @@ static void handle_assoc(struct hostapd_data *hapd,
 		p = hostapd_eid_supp_rates(hapd, mgmt->u.assoc_resp.variable);
 		/* Extended supported rates */
 		p = hostapd_eid_ext_supp_rates(hapd, p);
+		if (sta->flags && WLAN_STA_WME)
+			p = hostapd_eid_wme(hapd, p);
 		send_len += p - mgmt->u.assoc_resp.variable;
 
 		/* Request TX callback */
@@ -992,7 +1126,8 @@ static void handle_deauth(struct hostapd_data *hapd,
 
 
 static void handle_beacon(struct hostapd_data *hapd,
-			  struct ieee80211_mgmt *mgmt, size_t len)
+			  struct ieee80211_mgmt *mgmt, size_t len,
+			  struct hostapd_frame_info *fi)
 {
 	struct ieee802_11_elems elems;
 
@@ -1018,6 +1153,8 @@ static void handle_beacon(struct hostapd_data *hapd,
 		ieee802_11_sta_authenticate(hapd, NULL);
 	}
 
+	ap_list_process_beacon(hapd->iface, mgmt, &elems, fi);
+
 	if (!HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_EXCESSIVE))
 		return;
 
@@ -1033,6 +1170,30 @@ static void handle_beacon(struct hostapd_data *hapd,
 }
 
 
+static void handle_action(struct hostapd_data *hapd,
+			  struct ieee80211_mgmt *mgmt, size_t len)
+{
+	if (len < IEEE80211_HDRLEN + 1) {
+		hostapd_logger(hapd, mgmt->sa, HOSTAPD_MODULE_IEEE80211,
+			       HOSTAPD_LEVEL_DEBUG,
+			       "handle_action - too short payload (len=%lu)",
+			       (unsigned long) len);
+		return;
+	}
+
+	switch (mgmt->u.action.category) {
+	case WME_ACTION_CATEGORY:
+		hostapd_wme_action(hapd, mgmt, len);
+		return;
+	}
+
+	hostapd_logger(hapd, mgmt->sa, HOSTAPD_MODULE_IEEE80211,
+		       HOSTAPD_LEVEL_DEBUG,
+		       "handle_action - unknown action category %d",
+		       mgmt->u.action.category);
+}
+
+
 /**
  * ieee802_11_mgmt - process incoming IEEE 802.11 management frames
  * @hapd: hostapd BSS data structure (the BSS to which the management frame was
@@ -1046,17 +1207,21 @@ static void handle_beacon(struct hostapd_data *hapd,
  * addition, it can be called to re-inserted pending frames (e.g., when using
  * external RADIUS server as an MAC ACL).
  */
-void ieee802_11_mgmt(struct hostapd_data *hapd, u8 *buf, size_t len, u16 stype)
+void ieee802_11_mgmt(struct hostapd_data *hapd, u8 *buf, size_t len, u16 stype,
+		     struct hostapd_frame_info *fi)
 {
 	struct ieee80211_mgmt *mgmt = (struct ieee80211_mgmt *) buf;
 	int broadcast;
 
 	if (stype == WLAN_FC_STYPE_BEACON) {
 		HOSTAPD_DEBUG(HOSTAPD_DEBUG_EXCESSIVE, "mgmt::beacon\n");
-		handle_beacon(hapd, mgmt, len);
+		handle_beacon(hapd, mgmt, len, fi);
 		return;
 	}
 
+	if (fi && fi->passive_scan)
+		return;
+
 	broadcast = mgmt->bssid[0] == 0xff && mgmt->bssid[1] == 0xff &&
 		mgmt->bssid[2] == 0xff && mgmt->bssid[3] == 0xff &&
 		mgmt->bssid[4] == 0xff && mgmt->bssid[5] == 0xff;
@@ -1109,6 +1274,10 @@ void ieee802_11_mgmt(struct hostapd_data *hapd, u8 *buf, size_t len, u16 stype)
 		HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL, "mgmt::deauth\n");
 		handle_deauth(hapd, mgmt, len);
 		break;
+	case WLAN_FC_STYPE_ACTION:
+		HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL, "mgmt::action\n");
+		handle_action(hapd, mgmt, len);
+		break;
 	default:
 		hostapd_logger(hapd, mgmt->sa, HOSTAPD_MODULE_IEEE80211,
 			       HOSTAPD_LEVEL_DEBUG,
@@ -1219,6 +1388,14 @@ static void handle_assoc_cb(struct hostapd_data *hapd,
 			       "Could not add STA to kernel driver");
 	}
 
+	if (sta->flags & WLAN_STA_SHORT_PREAMBLE) {
+		hostapd_sta_set_flags(hapd, sta->addr,
+				      WLAN_STA_SHORT_PREAMBLE, ~0);
+	} else {
+		hostapd_sta_set_flags(hapd, sta->addr,
+				      0, ~WLAN_STA_SHORT_PREAMBLE);
+	}
+
 	wpa_auth_sm_event(sta->wpa_sm, WPA_ASSOC);
 	hostapd_new_assoc_sta(hapd, sta, !new_assoc);
 
