@@ -418,7 +418,7 @@ hostapd_acl_recv_radius(struct radius_msg *msg, struct radius_msg *req,
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL, "Re-sending authentication frame "
 		      "after successful RADIUS ACL query\n");
 	ieee802_11_mgmt(hapd, query->auth_msg, query->auth_msg_len,
-			WLAN_FC_STYPE_AUTH);
+			WLAN_FC_STYPE_AUTH, NULL);
 
  done:
 	if (prev == NULL)
