@@ -1,6 +1,6 @@
 /*
  * wpa_supplicant/hostapd - Default include files
- * Copyright (c) 2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2005-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -26,9 +26,11 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
+#ifndef _WIN32_WCE
 #include <signal.h>
 #include <sys/types.h>
 #include <errno.h>
+#endif /* _WIN32_WCE */
 #include <ctype.h>
 #include <time.h>
 
@@ -40,8 +42,10 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#ifndef __vxworks
 #include <sys/uio.h>
 #include <sys/time.h>
+#endif /* __vxworks */
 #endif /* CONFIG_NATIVE_WINDOWS */
 
 #endif /* INCLUDES_H */
