@@ -67,7 +67,7 @@ int os_get_random(unsigned char *buf, size_t len)
 	FILE *f;
 	size_t rc;
 
-	f = fopen("/dev/urandom", "r");
+	f = fopen("/dev/urandom", "rb");
 	if (f == NULL) {
 		printf("Could not open /dev/urandom.\n");
 		return -1;
@@ -134,3 +134,20 @@ int os_program_init(void)
 void os_program_deinit(void)
 {
 }
+
+
+int os_setenv(const char *name, const char *value, int overwrite)
+{
+	return setenv(name, value, overwrite);
+}
+
+
+int os_unsetenv(const char *name)
+{
+#if defined(__FreeBSD__) || defined(__NetBSD__)
+	unsetenv(name);
+	return 0;
+#else
+	return unsetenv(name);
+#endif
+}
