@@ -109,3 +109,15 @@ void os_program_deinit(void)
 	WSACleanup();
 #endif /* CONFIG_NATIVE_WINDOWS */
 }
+
+
+int os_setenv(const char *name, const char *value, int overwrite)
+{
+	return -1;
+}
+
+
+int os_unsetenv(const char *name)
+{
+	return -1;
+}
