@@ -73,15 +73,10 @@ static void rsn_preauth_receive(void *ctx, const u8 *src_addr,
 		return;
 	}
 	if (!sta && hdr->type == IEEE802_1X_TYPE_EAPOL_START) {
-		sta = wpa_zalloc(sizeof(struct sta_info));
+		sta = ap_sta_add(hapd, ethhdr->h_source);
 		if (sta == NULL)
 			return;
-		memcpy(sta->addr, ethhdr->h_source, ETH_ALEN);
 		sta->flags = WLAN_STA_PREAUTH;
-		sta->next = hapd->sta_list;
-		hapd->sta_list = sta;
-		hapd->num_sta++;
-		ap_sta_hash_add(hapd, sta);
 
 		ieee802_1x_new_station(hapd, sta);
 		if (sta->eapol_sm == NULL) {
