@@ -26,6 +26,8 @@
 #include "preauth.h"
 #include "radius_client.h"
 #include "driver.h"
+#include "beacon.h"
+#include "hw_features.h"
 #ifndef CONFIG_NATIVE_WINDOWS
 #include "hostap_common.h"
 #else /* CONFIG_NATIVE_WINDOWS */
@@ -111,6 +113,8 @@ static void ap_sta_hash_del(struct hostapd_data *hapd, struct sta_info *sta)
 
 void ap_free_sta(struct hostapd_data *hapd, struct sta_info *sta)
 {
+	int set_beacon = 0;
+
 	accounting_sta_stop(hapd, sta);
 	if (!(sta->flags & WLAN_STA_PREAUTH))
 		hostapd_sta_remove(hapd, sta->addr);
@@ -122,6 +126,32 @@ void ap_free_sta(struct hostapd_data *hapd, struct sta_info *sta)
 		hapd->sta_aid[sta->aid - 1] = NULL;
 
 	hapd->num_sta--;
+	if (sta->nonerp_set) {
+		sta->nonerp_set = 0;
+		hapd->iface->num_sta_non_erp--;
+		if (hapd->iface->num_sta_non_erp == 0)
+			set_beacon++;
+	}
+
+	if (sta->no_short_slot_time_set) {
+		sta->no_short_slot_time_set = 0;
+		hapd->iface->num_sta_no_short_slot_time--;
+		if (hapd->iface->current_mode->mode == HOSTAPD_MODE_IEEE80211G
+		    && hapd->iface->num_sta_no_short_slot_time == 0)
+			set_beacon++;
+	}
+
+	if (sta->no_short_preamble_set) {
+		sta->no_short_preamble_set = 0;
+		hapd->iface->num_sta_no_short_preamble--;
+		if (hapd->iface->current_mode->mode == HOSTAPD_MODE_IEEE80211G
+		    && hapd->iface->num_sta_no_short_preamble == 0)
+			set_beacon++;
+	}
+
+	if (set_beacon)
+		ieee802_11_set_beacons(hapd->iface);
+
 	eloop_cancel_timeout(ap_handle_timer, hapd, sta);
 
 	ieee802_1x_free_station(sta);
@@ -330,10 +360,10 @@ struct sta_info * ap_sta_add(struct hostapd_data *hapd, const u8 *addr)
 		return sta;
 
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL, "  New STA\n");
-	if (hapd->num_sta >= MAX_STA_COUNT) {
+	if (hapd->num_sta >= hapd->conf->max_num_sta) {
 		/* FIX: might try to remove some old STAs first? */
 		printf("  no more room for new STAs (%d/%d)\n",
-		       hapd->num_sta, MAX_STA_COUNT);
+		       hapd->num_sta, hapd->conf->max_num_sta);
 		return NULL;
 	}
 
