@@ -244,7 +244,7 @@ static ssize_t tls_pull_func(gnutls_transport_ptr ptr, void *buf,
 	}
 
 	end = conn->pull_buf + conn->pull_buf_len;
-	if (end - conn->pull_buf_offset < len)
+	if ((size_t) (end - conn->pull_buf_offset) < len)
 		len = end - conn->pull_buf_offset;
 	memcpy(buf, conn->pull_buf_offset, len);
 	conn->pull_buf_offset += len;
@@ -928,12 +928,16 @@ static int tls_connection_verify_peer(struct tls_connection *conn)
 
 u8 * tls_connection_handshake(void *ssl_ctx, struct tls_connection *conn,
 			      const u8 *in_data, size_t in_len,
-			      size_t *out_len)
+			      size_t *out_len, u8 **appl_data,
+			      size_t *appl_data_len)
 {
 	struct tls_global *global = ssl_ctx;
 	u8 *out_data;
 	int ret;
 
+	if (appl_data)
+		*appl_data = NULL;
+
 	if (in_data && in_len) {
 		if (conn->pull_buf) {
 			wpa_printf(MSG_DEBUG, "%s - %d bytes remaining in "
@@ -1157,9 +1161,10 @@ int tls_connection_set_master_key(void *ssl_ctx, struct tls_connection *conn,
 }
 
 
-int tls_connection_set_anon_dh(void *ssl_ctx, struct tls_connection *conn)
+int tls_connection_set_cipher_list(void *tls_ctx, struct tls_connection *conn,
+				   u8 *ciphers)
 {
-	/* TODO: set ADH-AES128-SHA cipher */
+	/* TODO */
 	return -1;
 }
 
